<?php

namespace Teknisa\PLA\Modules\Parametros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class RestricoesContratuais extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
	protected $parameterBagColumns;
    protected $dataSourceManager;
    protected $newCodeNT;
    protected $fileServerURL;
    protected $zhFileUploader;
    protected $util;
    protected $connection;
    protected $chaves;
    const TABLES = array("VIGFREQPRATMES", "VIGFREQOPCAGR", "VIGRESTPRODGR", "VIGLIMPESERGR", "VIGFREQPRODGRPRAT", "VIGFREQGRPRATMES");

    public function __construct($crudService, $parameterBag, $dataSourceManager, $newCodeNT, $fileServerURL, $zhFileUploader, $util, $connection) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCodeNT         = $newCodeNT;
        $this->fileServerURL     = $fileServerURL;
        $this->zhFileUploader    = $zhFileUploader;
        $this->util              = $util;
        $this->connection        = $connection;
        $this->chaves            = array();
    }

    //Utilities

    public function getTables() {
        return self::TABLES;
    }
    public function validateDuplicatedRecords(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->util->validateDuplicatedRecords($row, $row["dataSourceName"], $row["fieldsToValidate"]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function validateInterval(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if ($row["__is_new"]) {
                $this->validateRegisterVigencia($row);
            }
            if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                $nrvigencia = isset($row["NRVIGENCIA"]) ? $row["NRVIGENCIA"] : null;
                $this->validateIntervalOverlap($nrvigencia, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function validateRegisterVigencia($row) {
        $this->util->validateDuplicatedRecords($row, $row["dataSourceName"], $row["fieldsToValidate"]);
    }

    public function validateIntervalOverlap($vig, $start, $end, $tableName, $cdfilial, $idtiporestricao) {
        if (in_array(strtoupper($tableName), self::TABLES)) { // Preventing SQL injection
            $sql = "SELECT 1 FROM " . $tableName . 
                    " WHERE ((:DTVIGENCIA BETWEEN DTVIGENCIA AND DTVIGENCIAFIM) OR (:DTVIGENCIAFIM BETWEEN DTVIGENCIA AND DTVIGENCIAFIM))
                    AND (NRVIGENCIA <> :NRVIGENCIA OR :NRVIGENCIA IS NULL)
                    AND CDFILIAL= :CDFILIAL
                    AND IDTIPORESTRICAO = :IDTIPORESTRICAO
                    AND NRORG = :NRORG";
            $params = array(
                "NRVIGENCIA"    => $vig,
                "DTVIGENCIA"    => $start,
                "DTVIGENCIAFIM" => $end,
                "CDFILIAL"      => $cdfilial,
                "IDTIPORESTRICAO" => $idtiporestricao,
                "NRORG"         => $this->crudService->getEnvironment()->getNrOrg()
            );
            $dataSet = $this->connection->fetchAll($sql, $params);
            if (!empty($dataSet)) {
                throw new \Exception("intervalOverlap", 1);
            } 
        } else {
            throw new \Exception("tableNotFound", 1);
        }
    }

    public function getNewCode($newCodeKey, $newCodeLength = 10, $castNumber = false) {
        $nrorg = $this->crudService->getEnvironment()->getNrOrgTrab();
        $newCodeKey = strtoupper($newCodeKey);
        return $this->newCodeNT->getCodeByTableName($newCodeKey, $nrorg, 1, $newCodeLength, $castNumber);
    }

    public function buildConditions($columnNames, $values, $operators = null) {
        $conditions = array();

        foreach ($columnNames as $key => $columnName) {
            array_push($conditions, 
                array(
                    "columnName" => $columnName,
                    "operator" => is_array($operators) ? $operators[$key] : "=",
                    "value" => $values[$key]
                )
            );
        }
        return $conditions;
    }

    //Anexos
    private function setFileFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["DOCUMENTOANEXO"] !== null) {
				$row["DOCUMENTOANEXOFULLPATH"] = $this->fileServerURL.$row["DOCUMENTOANEXO"];
			}
		}
		return new DataSet("restr_contr_anexos", $rows);
	}

    public function anexosFind(DTO\Request $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $table = $conditions[1]["value"];
        $indexTable = -1;
        for($i = 0; $i < count($conditions); $i++){
            if($conditions[$i]['columnName'] == 'TABLE'){
                $indexTable = $i;
            }
        }
        if($indexTable != -1){
            array_splice($conditions, $indexTable, 1);
        }
        $dataSet = $this->crudService->getFilteredDataset($table, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $newDataSet = $this->setFileFullPath($dataSet);
        $response->addDataSet($newDataSet);
    }
    
    public function uploadFile(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->dataSourceName = $row{"TABLE"};
            foreach ($row["fileUpload"] as $value){
                if (isset($value) && isset($value["name"]) && isset($value["path"])) {
                    $row["DSDOCUMENTOANEXO"] = $value["name"];
                    $row["DOCUMENTOANEXO"] = $value["path"];
                } else {
                    throw new \Exception("errorSavingFile", 1);
                }
                $dataSet = new DataSet($this->dataSourceName, array($row));
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                parent::save($newRequest, $response);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteFile(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->dataSourceName = $rows[0]{"TABLE"};
            foreach ($rows as $row) {
                $this->zhFileUploader->deleteFile($row["DOCUMENTOANEXO"]);
            }
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function downloadFile(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $file = $row["DOCUMENTOANEXOFULLPATH"];
        $name = $row["DSDOCUMENTOANEXO"];
        $nameArray = explode(".", $name);
        $type = end($nameArray);
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        header("Content-Disposition: attachment; filename=" . $name);
        readfile($file);
        exit;
    }
    
    public function apagaVigencia($row){
        try{
             $conditions = array(
                array(
                    "columnName" => 'CDFILIAL',
                    "operator" => "=",
                    "value" => $row['CDFILIAL']
                ),
                array(
                    "columnName" => "IDTIPORESTRICAO",
                    "operator" => "=",
                    "value" => $row['IDTIPORESTRICAO']
                ),
                array(
                    "columnName" => "DTVIGENCIA",
                    "operator" => "=",
                    "value" => $row['DTVIGENCIA']
                ),
                array(
                    "columnName" => "DTVIGENCIAFIM",
                    "operator" => "=",
                    "value" => $row['DTVIGENCIAFIM']
                )
            );
            $dataSet = $this->crudService->getFilteredDataset(strtolower($row['DataSourceDestino']), $conditions);
            $padraoPrato = $dataSet->getRows();
            $notOverlap = true;
            
            if(!empty($padraoPrato)){
                $sql = "DELETE FROM ".$row['DataSourceDestino']." WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO AND DTVIGENCIA = :DTVIGENCIA AND DTVIGENCIAFIM = :DTVIGENCIAFIM";
                $query = $this->connection->prepare($sql);
                 
                $params = array(
                    "CDFILIAL"         => $row['CDFILIAL'],
                    "IDTIPORESTRICAO"  => $row['IDTIPORESTRICAO'],
                    "DTVIGENCIA"       => $row['DTVIGENCIA'],
                    "DTVIGENCIAFIM"    => $row['DTVIGENCIAFIM']
                ); 
                $query->execute($params);
            }else{
                $sql =  "SELECT 1 FROM " . $row['DataSourceDestino'] . 
                        " WHERE ((:DTVIGENCIA BETWEEN DTVIGENCIA AND DTVIGENCIAFIM) 
                             OR (:DTVIGENCIAFIM BETWEEN DTVIGENCIA AND DTVIGENCIAFIM)
                             OR (:DTVIGENCIA = DTVIGENCIA) )
                            AND (NRVIGENCIA <> :NRVIGENCIA OR :NRVIGENCIA IS NULL)
                            AND CDFILIAL= :CDFILIAL
                            AND IDTIPORESTRICAO = :IDTIPORESTRICAO
                            AND NRORG = :NRORG";
                $params = array(
                    "NRVIGENCIA"       => null,
                    "CDFILIAL"         => $row['CDFILIAL'],
                    "IDTIPORESTRICAO"  => $row['IDTIPORESTRICAO'],
                    "DTVIGENCIA"       => $row['DTVIGENCIA'],
                    "DTVIGENCIAFIM"    => $row['DTVIGENCIAFIM'],
                    "NRORG"            => $this->crudService->getEnvironment()->getNrOrg()
                );
                $dataSet = $this->connection->fetchAll($sql, $params);
                $notOverlap = empty($dataSet);
            }
            return !empty($padraoPrato) || $notOverlap;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function retornaServicosPorFilial($nrvigencia, $cdfilial, $tableName){
        $sql = "SELECT
            FIL.CDFILIAL,
            FIL.CDFILIAL || ' | ' || FIL.NMFILIAL AS NMFILIAL,
            S.CDSERVFILI,
            S.NMSERVICO AS NMSERVICO,
            TN.NR".$tableName."
        FROM
            ".$tableName."   TN
            JOIN SERVICO         S ON TN.CDSERVFILI = S.CDSERVFILI
                              AND TN.CDFILIAL = S.CDFILIAL
            JOIN FILIAL          FIL ON FIL.CDFILIAL = S.CDFILIAL
                               AND TN.CDFILIAL = FIL.CDFILIAL
        WHERE
            TN.NRVIGENCIA = :NRVIGENCIA
            AND TN.CDFILIAL = :CDFILIAL";
        $params = array(
            "NRVIGENCIA"    => $nrvigencia,
            "CDFILIAL"      => $cdfilial
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function retornaServicosRestrProd($nrvigencia, $cdfilial){
        $sql = "SELECT
            FIL.CDFILIAL,
            FIL.CDFILIAL || ' | ' || FIL.NMFILIAL AS NMFILIAL,
            S.CDSERVFILI,
            S.NMSERVICO AS NMSERVICO,
            RP.CDPRODUTO,
            RP.NRGRUPOPRATO,
            RP.NRRESTPRODGR,
            RP.NRGRUPOPRATOSERV
        FROM
            RESTPRODGR  RP
            JOIN SERVICO         S ON RP.CDSERVFILI = S.CDSERVFILI
                              AND RP.CDFILIAL = S.CDFILIAL
            JOIN FILIAL          FIL ON FIL.CDFILIAL = S.CDFILIAL
                               AND RP.CDFILIAL = FIL.CDFILIAL
        WHERE
            RP.NRVIGENCIA = :NRVIGENCIA
            AND RP.CDFILIAL = :CDFILIAL";
        $params = array(
            "NRVIGENCIA"    => $nrvigencia,
            "CDFILIAL"      => $cdfilial
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
        public function retornaItemNr($row, $item, $tableName){
        if(isset($this->chaves[$item['NRGRUPOPRATOSERV']])){
            return $this->chaves[$item['NRGRUPOPRATOSERV']];
        }else{
            $sql = "SELECT GR.CDGRUPOPRATO,
                           GR.NMGRUPOPRATO,
                           GPS.NRGRUPOPRATOSERV
                      FROM GRUPOPRATOSERV GS 
                           INNER JOIN GRUPOPRATO GR ON GR.NRGRUPOPRATO = GS.NRGRUPOPRATO
                           LEFT JOIN GRUPOPRATO GD ON GD.CDGRUPOPRATO = GR.CDGRUPOPRATO AND GD.CDFILIAL = :CDFILIALDEST
                           LEFT JOIN GRUPOPRATOSERV GPS ON GPS.CDSERVFILI = GS.CDSERVFILI AND GPS.CDSERVFILI = :CDSERVFILI AND GPS.NRGRUPOPRATO = GD.NRGRUPOPRATO
                     WHERE GR.CDFILIAL = :CDFILIALORIG
                       AND GS.CDSERVFILI = :CDSERVFILI
                       AND EXISTS (SELECT 1 FROM ".$tableName." IT WHERE IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                                                     AND IT.NRGRUPOPRATOSERV = :NRGRUPOPRATOSERVORIG
                                                                     AND IT.NRGRUPOPRATO = GR.NRGRUPOPRATO)";
            $params = array(
                "CDFILIALDEST"            => $row['CDFILIAL'],
                "CDFILIALORIG"            => $row['CDFILIALORIG'],
                "CDSERVFILI"              => $item['CDSERVFILI'],
                "NRGRUPOPRATOSERVORIG"    => $item['NRGRUPOPRATOSERV']
            );
            $dataSet = $this->connection->fetchAll($sql, $params);
            if(empty($dataSet)){
                return null;
            }else{
                $this->chaves[$item['NRGRUPOPRATOSERV']] = $dataSet[0];
                return $dataSet[0];
            }
        }
    }
}